package org.n52.ifgicopter.spf;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.n52.ifgicopter.spf.common.IModule;
import org.n52.ifgicopter.spf.gui.PluginGUI;
import org.n52.ifgicopter.spf.input.IInputPlugin;


/**
 * A template for an IInputPlugin implementation.
 * 
 * @author matthes rieke <m.rieke@52north.org>
 *
 */
public class TemplateInputPlugin implements IInputPlugin {

	private TemplateWorkerThread workerThread;

	private List<Map<String, Object>> buffer = new ArrayList<Map<String,Object>>();


	@Override
	public void init() throws Exception {
		/*
		 * here you could initialized resource
		 * you need for your plugin to work.
		 * 
		 * For instance, you should start a working
		 * thread here.
		 * Could also be the initialization of
		 * a Serial Port (which perhaps manages its own threads).
		 */
		this.workerThread = new TemplateWorkerThread();
		new Thread(this.workerThread).start();
	}

	@Override
	public void shutdown() throws Exception {
		/*
		 * here you should free all used
		 * resources again
		 */
		this.workerThread.stopThread();
	}

	@Override
	public InputStream getConfigFile() {
		/*
		 * This method must return the Plugin description file
		 * as an instance of the project-inlined schema
		 * "spf-plugin_sensorML.xsd".
		 * It must be provided as an InputStream for convenience. 
		 */
		FileInputStream is = null;
		try {
			is = new FileInputStream(new File("input-template.xml"));
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		return is;
	}

	@Override
	public String getName() {
		/*
		 * return a human readable name for the plugin
		 */
		return "TemplateInputPlugin";
	}

	@Override
	public List<Map<String, Object>> getNewData() {
		/*
		 * this is where the actual data is transmitted.
		 */
		List<Map<String, Object>> copy;
		synchronized (this.buffer) {
			copy = new ArrayList<Map<String,Object>>(this.buffer);
			this.buffer.clear();
		}
		return copy;
	}

	@Override
	public int getStatus() {
		/*
		 * inform the framework about our state
		 */
		return IModule.STATUS_RUNNING;
	}

	@Override
	public String getStatusString() {
		/*
		 * Human readable version of the status
		 */
		return "Everything as expected.";
	}

	@Override
	public PluginGUI getUserInterface() {
		/*
		 * if you want your plugin to provide a user interface,
		 * return an instance of PluginGUI here. Else, return null.
		 */
		return null;
	}

	@Override
	public boolean hasNewData() {
		/*
		 * This method must return true, if new data is available
		 * for processing
		 */
		boolean result = false;
		synchronized (this.buffer) {
			result = !this.buffer.isEmpty();
		}
		return result;
	}
	
	
	
	
	private class TemplateWorkerThread implements Runnable {

		private boolean running = true;
		private Object sleepMonitor = new Object();

		@Override
		public void run() {
			/*
			 * process data of serial ports or
			 * other ways of data provision
			 */
			while (this.running) {
				
				/*
				 * sleep/wait e.g. 2 seconds
				 * do on a monitor to react quick
				 * for shutdown calls.
				 */
				synchronized (this.sleepMonitor) {
					try {
						this.sleepMonitor.wait(2000);
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
				}
				
				if (!this.running) return;
				
				/*
				 * do the actual processing
				 */
				Map<String,Object> data = new HashMap<String, Object>();
				data.put("temperature", 24.0);
				data.put("humidity", 89.3);
				
				synchronized (TemplateInputPlugin.this.buffer) {
					TemplateInputPlugin.this.buffer.add(data);
				}
			}
			
			System.out.println("We are all done.");
		}
		
		public void stopThread() {
			this.running = false;
			synchronized (this.sleepMonitor) {
				this.sleepMonitor.notifyAll();
			}
		}
		
	}

}
